# -*- coding: iso-8859-15 -*-
#    Share of attention to (100-x) % least popular works
#    function of Zipf's law parameter and 100 universe sizes
#
#    To generate figure A-4 of "Sharing" on has to run:
#    python figureA-4.py 1.0 20.0
#
#    Copyright (C) 2006-2009 - Philippe Aigrain
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

from pylab import *

def harmonic(s,n):
  k=0.0;
  for i in arange (1.0,n+1.0,1.0):
    k += pow(i,-s);
  return k;  

def main(argv):
  if len(argv) < 3:
    print ("Usage: %s <given parameter of Zipf's law><percentage of excluded most popular works>") % argv[0]
    print "Shows the percentage of access that go to all but the excluded works depending on the size of the universe for a given parameter"
    sys.exit(0)
  a = float(argv[1])
  excluded = float(argv[2])/100.0
# K is the step for increment in sizes of universe
  K = 1000
# N is the number of universe sizes (ex. for 100, size goes from 1000 to 1000000)
  N = 100
  # vectors for results
  s = zeros(N,float)
  # vector for cumulated distribution for the given a
  cum = zeros(N*K, float)
  n = 0
  # for all sizes
  for siz in arange(K,K*(N+1),K):
    # computing the cumulated distribution and its sum
    for j in arange(siz,int(siz*excluded)-1,-1):
      lastj = j
      if j<siz:
        cum[j-1] = cum[j] + float(j)**(-a)
      else:
	cum[j-1] = float(j)**(-a)
    s[(siz/K)-1] = cum[lastj] / harmonic(a,siz)
  plotpar = {'axes.labelsize': 24,
               'text.fontsize': 24,
               'legend.fontsize': 20,
               'xtick.labelsize': 22,
               'ytick.labelsize': 22,
               'text.usetex': False}
  rcParams.update(plotpar)
  close(1)
  figure(num=1, figsize=(16,12), facecolor='w', edgecolor='k')
# Plotting
# Replace by next line for BW figure
# plot(arange(K,K*(N+1),K),s,'k-',lw=2)
# Replace by next line for a color figure
  plot(arange(K,K*(N+1),K),s,'b-',lw=2)
  xlabel('Number of works')
  xticks((20000,40000,60000,80000,100000),('20000','40000','60000','80000','100000'))
  ylabel('Access to %d pc. least popular for a Zipf\'s law with param %3.1f' % (100.0-(excluded*100.0),a))
  grid(True)
  savefig('illustration.png')
  show()

if __name__ == '__main__':
    main(sys.argv)
